

/**
 * SVG Animation created by VISONE ( http://visone.info )
 * Compatible with any browser that supports Javascript and SVG animations.
 * 
 * author: mheine, Universitiaet Konstanz
 * 
 * Note: Elements has following properties:
 * Node: (e.g.) id = "y.node.g0.node1" where g0 is the graph containing the node (here graph 0)
 *          and node1 is the original node-id in Visone.
 *      (e.g.) class = "g0 ellipse unselected" where g0 is the graph containing the node (here graph 0),
 *          ellipse is the shape and unselected says the node is unselected
 * 
 * Edge: (e.g.) id = "y.edge.g0.1.1-3" where g0 is the graph containing the edge,
 *          1-3 is the direction of the edge (here from node 1 to node 3) 
 *          and 1 is the number of the edge (if there are more edges between node 1 and node 3)
 *          
 * */

/*- svg document -*/
var document;

/*- support boolean -*/
supportChecked = false;

/*- source and target graph -*/
var sourceNo = 0;
var targetNo = 1;
var noOfGraphs = 0;

/*- control booleans -*/
var startLooping = false;
var playing = false;
var run = false;
var stop = false;
var onclick = false;
var controls = true;

/*- highlight edges variables -*/
var highlight = true;
var edgeColors;
var changedEdges;
var fadeOutColor = 'rgb(202,0,32)';
var fadeInColor = 'rgb(5,113,176)';

/*- loop checkbox -*/
var loop = false;

/*- edges and nodes of source graph -*/
var nodesFrom;
var edgesFrom;

/*- edges and nodes of target graph -*/
var nodesTo;
var edgesTo;

/*- arrays of animation elements -*/
var animateElements;
var animateMoveElements;
var animateHideElements;
var animateShowElements;

/*- slider events variables -*/
var sliderActive = false;
var speedVal = false;
var delayVal = false;
var graphVal = false;
var sliderBoundings;
var thumb;
var graphThumb;
var thumbPosition = 0;
var slideDuration = 2;
var slideDelay = 0;
var slideBeginElementDelay = 0;
var duration = 2;
var delay = 0;
var beginElementDelay = 0;

/*- control variables - */
var barColor = '#dddddd';
var buttonColor = '#808080';
var inactiveButtonColor = '#9e9e9e';
var buttonRadius = '15';
var playD = 'M0,0 l-15,9 l0,-18 z';
var nextD = 'M0,0 l-8.5,9 l0,-18 l8.5,9 l0,-9 l8.5,9 l-8.5,9 z';
var prevD = 'M0,0 l8.5,9 l0,-9 l8.5,9 l0,-18 l-8.5,9 l0,-9 z';
var stopD = 'M0,0 l-16,0 l0,-16 l16,0 z';

var sliderStrokesD = 'M0,13 0,20 m10,0 0,-3 m10,-4 0,7 m10,0 0,-3 m10,-4 0,7 m10,0 0,-3 m10,-4 0,7 m10,0 0,-3 m10,-4 0,7 m10,0 0,-3 m10,-4 0,7 m10,0 0,-3 m10,-4 0,7 m10,0 0,-3 m10,-4 0,7';
var sliderLineD = 'M0,6 l140,0';
var thumbD = 'M-5,0 l0,10 l4,3 l4,-3 l0,-10 z';
var thumbColor = '#dddddd';
var thumbStroke = 'grey';
var graphSliderLine = 'M0,6 l140,0';
var graphSliderStrokesBegin = 'M0,12 0,0';
var graphSliderStrokesEnd = 'M140,12 140,0';
var graphSliderStep = 0;
var graphSliderLength = 140;
var graphStrokeLength = 12;

/*- Visone link -*/
var visoneLine1 = 'Created with';
var visoneLine2 = 'Visone';
var visoneLink = 'http://visone.info';
var visoneCreated = false;

/*- font -*/
var font = 'Helvetica';

/*- x-positions relative to P(docWidth/2, 0) -*/
var xNoOfGraphsField = -270;
var yNoOfGraphsField = 90;
var xDelaySlider = 30;
var yDelaySlider = 60;
var xSpeedSlider = 30;
var ySpeedSlider = 10;
var xPlayPrevNext = -260;
var yPlayPrevNext = 25;
var xHighlightEdges = 220;
var yHighlightEdges = 17;
var xJumpTo = -120;
var yJumpTo = 4;
var xLoop = 220;
var yLoop = 60;
var xGraphSlider = -330;
var yGraphSlider = 60;


/*----------------------------------------------------------------------------------*/
/*--------------------------Start-and-Stop-of-Animation-----------------------------*/
/*----------------------------------------------------------------------------------*/

/*----- starts the animation -----*/
function play(evt) {
    if(run || noOfGraphs <= 1){
        return;
    }
    
    document = evt.target.ownerDocument;
    
    changeButtonColor(false, false);
    
    var playButton = document.getElementById('playButton');
    var paths = playButton.getElementsByTagName('path');
    
    paths.item(0).setAttributeNS(null, 'opacity', '0');
    paths.item(1).setAttributeNS(null, 'opacity', '1');
    playButton.setAttributeNS(null, 'onclick', 'freeze(true)');
    stop = false;
    playing = true;
    run = true;
    targetNo = sourceNo + 1;
    
    initialize();
    startHelp();
}

/*----------------------------------------------------------------------------------*/

/*----- animates to previous graph -----*/
function previous(evt) {
    if(run){
        return;
    }
    
    document = evt.target.ownerDocument;
    var graphNo = sourceNo - 1;
    
    if(graphNo < 0){
        graphNo = noOfGraphs - 1;
    }
    
    jumpToGraph(graphNo);
}

/*----------------------------------------------------------------------------------*/

/*----- animates to next graph -----*/
function next(evt) { 
    if(run){
        return;
    }
    document = evt.target.ownerDocument;
    var graphNo = sourceNo + 1;
    
    if(graphNo == noOfGraphs){
        graphNo = 0;
    }
    
    jumpToGraph(graphNo);
}

/*----------------------------------------------------------------------------------*/

/*----- stops the animation -----*/
function freeze(setInactive){
    var playButton = document.getElementById('playButton');
    var playCircle = playButton.getElementsByTagName('circle').item(0);
    if(setInactive){
    	playCircle.setAttributeNS(null, 'fill', inactiveButtonColor);
    }
    
    var paths = playButton.getElementsByTagName('path');
    paths.item(0).setAttributeNS(null, 'opacity', '1');
    paths.item(1).setAttributeNS(null, 'opacity', '0');
    playButton.setAttributeNS(null, 'onclick', 'play(evt)');
    stop = true;
}

/*----------------------------------------------------------------------------------*/

/*----- jumps to given graph -----*/
function jumpToEvent(evt){
    var inputField = document.getElementById('inputField');
    input = parseInt(inputField.value);
    if(isNaN(input) || run || sourceNo == input-1 || input < 1 || input > noOfGraphs){
        return;
    }
    
    jumpToGraph(input-1);
}

/*----------------------------------------------------------------------------------*/

/*----- jumps to given graph -----*/
function jumpToGraph(target){   
    targetNo = target;
    run = true;
    
    initialize();
    changeButtonColor(false, true);
    
    duration = slideDuration;
    delay = slideDelay;
    beginElementDelay = slideBeginElementDelay;
    
    if(duration == 0){
        switchGraph();
    }
    else{
        fadeOut();
    } 
    
}

/*----------------------------------------------------------------------------------*/

function changeButtonColor(activatedColor, changeStopButton){
    if(activatedColor){
        var nextButton = document.getElementById('nextButton');   
        var nextCircle = nextButton.getElementsByTagName('circle').item(0);
        nextCircle.setAttributeNS(null, 'fill', buttonColor);
        var prevButton = document.getElementById('prevButton');
        var prevCircle = prevButton.getElementsByTagName('circle').item(0);
        prevCircle.setAttributeNS(null, 'fill', buttonColor);
        var playButton = document.getElementById('playButton');
        var playCircle = playButton.getElementsByTagName('circle').item(0);
        playCircle.setAttributeNS(null, 'fill', buttonColor);
        var jumpButton = document.getElementById('jumpButton');
        var jumpRect = jumpButton.getElementsByTagName('rect').item(0);
        jumpRect.setAttributeNS(null, 'fill', buttonColor);
    }
    else{
        var nextButton = document.getElementById('nextButton');   
        var nextCircle = nextButton.getElementsByTagName('circle').item(0);
        nextCircle.setAttributeNS(null, 'fill', inactiveButtonColor);
        var prevButton = document.getElementById('prevButton');
        var prevCircle = prevButton.getElementsByTagName('circle').item(0);
        prevCircle.setAttributeNS(null, 'fill', inactiveButtonColor);
        var jumpButton = document.getElementById('jumpButton');
        var jumpRect = jumpButton.getElementsByTagName('rect').item(0);
        jumpRect.setAttributeNS(null, 'fill', inactiveButtonColor);
        
        if(changeStopButton){
            var playButton = document.getElementById('playButton');
            var playCircle = playButton.getElementsByTagName('circle').item(0);
            playCircle.setAttributeNS(null, 'fill', inactiveButtonColor);
        }  
    }
}

/*----------------------------------------------------------------------------------*/

/*----- starts animation if there are no controls -----*/

function animateWithoutControls(evt, isOnclick, isHighlight, speedDouble, delayDouble){  
    if(isOnclick && run){
        stop = true;
        return;
    }
    
    document = evt.target.ownerDocument;
    
    /*- inserts Visone link -*/
    if(!visoneCreated){
        insertVisone();
    }
    
    
    slideDelay = delayDouble;
    slideDuration = speedDouble;
    highlight = isHighlight;
    onclick = isOnclick;
    controls = false;
    playing = true;
    run = true;
    
    initialize();
    if(onclick){
        startHelp();
    }
    else{
        setTimeout(startHelp, 1000);
    }
}

/*----------------------------------------------------------------------------------*/
/*------------------------------------Animation-------------------------------------*/
/*----------------------------------------------------------------------------------*/

/*----- needed to loop over graphs -----*/
function startHelp() {
    deleteAnimations();
    
    if (startLooping && loop) {
        //startLooping = false;
        sourceNo = 0;
        targetNo = 1;
        switchGraph();
    }
    if(stop || ((targetNo == noOfGraphs) && !loop)){
        run = false;
        playing = false;
        stop = false;
        if(!onclick){
            changeButtonColor(true, true);
        }
        
        freeze(false);
        
        return;
    }
    
    initialize();

    if (nodesTo.length == 0 && targetNo == 1) {
        return;
    }
    else if (nodesTo.length == 0) {
        startLooping = true;
        targetNo = 0;
        nodesTo = document.getElementsByClassName('g' + targetNo);
        edgesTo = document.getElementsByClassName('e.g' + targetNo);
    }
    
    duration = slideDuration;
    delay = slideDelay;
    beginElementDelay = slideBeginElementDelay;
    
    if(duration == 0){
        switchGraph();
    }
    else{
        fadeOut();
    }
};

/*----------------------------------------------------------------------------------*/

/*----- assigns source and targetgraph -----*/
function initialize() {
    nodesFrom = document.getElementsByClassName('g' + sourceNo);
    nodesTo = document.getElementsByClassName('g' + targetNo);
    edgesFrom = document.getElementsByClassName('e.g' + sourceNo);
    edgesTo = document.getElementsByClassName('e.g' + targetNo);

    animateMoveElements = new Array();
    animateHideElements = new Array();
    animateShowElements = new Array();
    animateElements = new Array();
    edgeColors = new Array();
    changedEdges = new Array();
}

/*----------------------------------------------------------------------------------*/

/* deletes animation-elements so all changes caused by these elementes are undone*/
function deleteAnimations() {
    for (var i = 0; i < animateElements.length; i++) {
        document.documentElement.removeChild(animateElements[i]);
    }

    animateElements = new Array();
}

/*----------------------------------------------------------------------------------*/

/*----- hides elements of the source graph-----*/
function fadeOut() {
    for (var i = 0; i < nodesFrom.length; i++) {
        var nodeFromId = nodesFrom.item(i).id;

        if (nodeFromId == "empty") {
            continue;
        }

        var temp = nodeFromId.split('.');
        var nodeId = temp[3];
        
        for(k=4; k<temp.length; k++){
            nodeId += '.'+temp[k];
        }
        
        var str = 'y.node.g' + targetNo + '.' + nodeId;
        var nodeTo = document.getElementById(str);

        /*----- if the node isn't in the target-graph or switching from the last to the first graph fade it out -----*/

        if (nodeTo == null || startLooping) {
            var animateHideNode = createAnimateElement('hide.' + nodesFrom.item(i).id, nodesFrom.item(i).id, 'opacity', '1', '0', '1');
            animateHideElements.push(animateHideNode);
        }
        else {
            createMoveNodeAnimation(nodesFrom.item(i), nodeTo);
            createResizeNodeAnimation(nodesFrom.item(i), nodeTo);
            createColorNodeAnimation(nodesFrom.item(i), nodeTo);
        }
    }

    for (var i = 0; i < edgesFrom.length; i++) {
        var edgeFrom = edgesFrom.item(i);
        var edgeFromId = edgeFrom.id;

        if (edgeFromId == "empty") {
            continue;
        }

        var temp = edgeFromId.split('.');
        var edgeId = temp[4];
        
        for(k=5; k<temp.length; k++){
            edgeId += '.'+temp[k];
        }
        
        var str = 'y.edge.g' + targetNo + '.' + temp[3] + '.' + edgeId;
        var edgeTo = document.getElementById(str);
        var hideEdge = true;
        var ending = temp[3] + '.' + edgeId;

        if (edgeTo == null || startLooping) {
            var subtemp = edgeId.split('-');
            ending = subtemp[1] + '-' + subtemp[0];
            str = 'y.edge.g' + targetNo + '.' + ending;
            edgeTo = document.getElementById(str);
        }

        if (edgeTo != null) {
            hideEdge = false;
            var classesTo = edgeTo.getAttribute('class').split(' ');
            var classesFrom = edgesFrom[i].getAttribute('class').split(' ');

            if ((classesFrom[1] == 'directed' && classesTo[1] == 'none') || (classesFrom[2] != classesTo[2])) {
                hideEdge = true;
            }
            else if ((classesFrom[3] == 'unconfirmed' && classesTo[3] == 'none') || (classesFrom[4] != classesTo[4])) {
                hideEdge = true;
            }
        }

        if (hideEdge) {
            if(highlight && !startLooping){
                var edgeGroup = edgeFrom.childNodes[1];
                edgeColors.push(edgeGroup.getAttributeNS(null, 'stroke'));
                changedEdges.push(edgeGroup);
                edgeGroup.setAttributeNS(null, 'stroke', fadeOutColor);
            }
            
            var animateHideEdge = createAnimateElement('hide.' + edgesFrom.item(i).id, edgesFrom.item(i).id, 'opacity', '1', '0', '1');
            animateHideElements.push(animateHideEdge);
        }
        else {
            createMoveEdgeAnimation(edgesFrom.item(i), edgeTo, ending);
        }
    }

    /*----- starts animation elements -----*/
   
    for (var i = 0; i < animateHideElements.length; i++) {
        animateHideElements[i].beginElementAt(beginElementDelay);
    }
    
    if (animateHideElements.length == 0) {
        setTimeout(move, 0);
    }
    else {
        setTimeout(move, 1000 * (duration + beginElementDelay));
    }
}

/*----------------------------------------------------------------------------------*/

/*----- creates animation element to move a node -----*/
function createMoveNodeAnimation(nodeFrom, nodeTo) {

    /*----- attribute class has form 'graph shape [un]selected' -----*/
    var tagName = null;
    var tagNameSource = nodeFrom.getAttribute('class').split(' ')[1];
    var tagNameTarget = nodeTo.getAttribute('class').split(' ')[1];

    if (tagNameSource == 'ellipse') {
        tagName = 'ellipse';
    }
    else if (tagNameSource == 'rectangle' || tagNameSource == '3Drectangle') {
        tagName = 'rect';
    }
    else {
        tagName = 'path';
    }

    var nodeFromMove = nodeFrom.getElementsByTagName(tagName).item(0);
    if (tagName == 'rect' && nodeFrom.getAttribute('class').split(' ')[2] == 'selected') {
        nodeFromMove = nodeFrom.getElementsByTagName(tagName).item(8);
    }

    if (tagNameTarget == 'ellipse') {
        tagName = 'ellipse';
    }
    else if (tagNameTarget == 'rectangle' || tagNameTarget == '3Drectangle') {
        tagName = 'rect';
    }
    else {
        tagName = 'path';
    }

    var nodeToMove = nodeTo.getElementsByTagName(tagName).item(0);
    if (tagName == 'rect' && nodeTo.getAttribute('class').split(' ')[2] == 'selected') {
        nodeToMove = nodeTo.getElementsByTagName(tagName).item(8);
    }

    var xSource = 0;
    var xTarget = 0;
    var ySource = 0;
    var yTarget = 0;
    var diffX = 0;
    var diffY = 0;

    if (tagNameSource == 'ellipse') {
        xSource = nodeFromMove.getAttribute('cx');
        ySource = nodeFromMove.getAttribute('cy');
    }
    else if (tagNameSource == 'rectangle' || tagNameSource == '3Drectangle') {
        xSource = nodeFromMove.getAttribute('x');
        ySource = nodeFromMove.getAttribute('y');
    }
    else {
        var coordinates = nodeFromMove.getAttribute('d').split(' ');
        var sumX = 0;
        var sumY = 0;
        var count = 0;

        for (var j = 0; j < coordinates.length; j++) {
            if (j % 2 == 0) {
                if (j != (coordinates.length - 1)) {
                    sumX += parseFloat(coordinates[j].substr(1, coordinates[j].length));
                    count++;
                }
            }
            else {
                sumY += parseFloat(coordinates[j]);
            }
        }

        xSource = sumX / count;
        ySource = sumY / count;
    }

    if (tagNameTarget == 'ellipse') {
        xTarget = nodeToMove.getAttribute('cx');
        yTarget = nodeToMove.getAttribute('cy');
    }
    else if (tagNameTarget == 'rectangle' || tagNameTarget == '3Drectangle') {
        xTarget = nodeToMove.getAttribute('x');
        yTarget = nodeToMove.getAttribute('y');
    }
    else {
        var coordinates = nodeToMove.getAttribute('d').split(' ');
        
        var count = 0;
        var sumX = 0;
        var sumY = 0;

        for (var j = 0; j < coordinates.length; j++) {
            if (j % 2 == 0) {
                if (j != (coordinates.length - 1)) {
                    sumX += parseFloat(coordinates[j].substr(1, coordinates[j].length));
                    count++;
                }
            }
            else {
                sumY += parseFloat(coordinates[j]);
            }
        }

        xTarget = sumX / count;
        yTarget = sumY / count;
    }

    var factor = nodeFrom.getElementsByTagName('g').item(0).getCTM().a;

    diffX = (xTarget - xSource) * factor;
    diffY = (yTarget - ySource) * factor;

    /*----- Check if node has to be moved -----*/
    if (diffX == 0 && diffY == 0) {
        return;
    }

    var animateMoveNode = createAnimateTransformElement('tran.' + nodeFrom.id, nodeFrom.id, 'translate', 'transform', '0,0', diffX + ',' + diffY, 'replace', '1');
    animateMoveElements.push(animateMoveNode);
}

/*----------------------------------------------------------------------------------*/

/*----- creates animation element to resize a node -----*/
function createResizeNodeAnimation(nodeFrom, nodeTo) {
    var tagNameSource = nodeFrom.getAttribute('class').split(' ')[1];
    var tagNameTarget = nodeTo.getAttribute('class').split(' ')[1];

    if (tagNameSource != tagNameTarget) {
        return;
    }

    if (tagNameSource == 'ellipse') {
        var ellSource = nodeFrom.getElementsByTagName('ellipse')[0];
        var borderSource = nodeFrom.getElementsByTagName('ellipse')[1];
        var ellTarget = nodeTo.getElementsByTagName('ellipse')[0];
        var yRadiusFrom = ellSource.getAttributeNS(null, 'ry');
        var xRadiusFrom = ellSource.getAttributeNS(null, 'rx');
        var yRadiusTo = ellTarget.getAttributeNS(null, 'ry');
        var xRadiusTo = ellTarget.getAttributeNS(null, 'rx');
        var changeBorder = false;

        if (borderSource != null && borderSource.getAttributeNS(null, 'fill-opacity') != "0") {
            changeBorder = true;
        }

        /*----- checks if node has to be resized -----*/
        if (xRadiusFrom != xRadiusTo) {
            ellSource.setAttributeNS(null, 'id', 'ell.' + nodeFrom.id);
            var animateResize = createAnimateElement('res.1.' + ellSource.id, ellSource.id, 'rx', xRadiusFrom, xRadiusTo, '1');
            animateMoveElements.push(animateResize);

            if (changeBorder) {
                borderSource.setAttributeNS(null, 'id', 'bor.' + nodeFrom.id);
                animateResize = createAnimateElement('res.1.' + borderSource.id, borderSource.id, 'rx', xRadiusFrom, xRadiusTo, '1');
                animateMoveElements.push(animateResize);
            }
        }
        if (yRadiusFrom != yRadiusTo) {
            ellSource.setAttributeNS(null, 'id', 'ell.' + nodeFrom.id);
            var animateResize = createAnimateElement('res.2.' + ellSource.id, ellSource.id, 'ry', yRadiusFrom, yRadiusTo, '1');
            animateMoveElements.push(animateResize);

            if (changeBorder) {
                borderSource.setAttributeNS(null, 'id', 'bor.' + nodeFrom.id);
                animateResize = createAnimateElement('res.2.' + borderSource.id, borderSource.id, 'ry', yRadiusFrom, yRadiusTo, '1');
                animateMoveElements.push(animateResize);
            }
        }
    }
    else if (tagNameSource == 'rectangle' || tagNameSource == '3Drectangle') {
        var sourceNodes = nodeFrom.getElementsByTagName('rect');
        var rectSource = sourceNodes[0];
        var borderSource = sourceNodes[1];
        var rectTarget = nodeTo.getElementsByTagName('rect')[0];
        var heightFrom = rectSource.getAttributeNS(null, 'height');
        var widthFrom = rectSource.getAttributeNS(null, 'width');
        var heightTo = rectTarget.getAttributeNS(null, 'height');
        var widthTo = rectTarget.getAttributeNS(null, 'width');
        var changeBorder = false;

        if (borderSource != null && borderSource.getAttributeNS(null, 'fill-opacity') != "0") {
            changeBorder = true;
        }

        /*----- checks if node has to be resized -----*/
        if (heightFrom != heightTo) {
            rectSource.setAttributeNS(null, 'id', 'rect.' + nodeFrom.id);
            var animateResize = createAnimateElement('res.1.' + rectSource.id, rectSource.id, 'height', heightFrom, heightTo, '1');
            animateMoveElements.push(animateResize);

            if (tagNameSource == '3Drectangle') {
                /*sourceNodes[1].setAttributeNS(null, 'id', 'bor.1' + nodeFrom.id);
                 animateResize = createAnimateElement('res.1.' + sourceNodes[1].id, sourceNodes[1].id, 'height', heightFrom, heightTo, '1');
                 animateMoveElements.push(animateResize);

                 sourceNodes[2].setAttributeNS(null, 'id', 'bor.2' + nodeFrom.id);
                 animateResize = createAnimateElement('res.1.' + sourceNodes[2].id, sourceNodes[2].id, 'width', widthFrom, widthTo, '1');
                 animateMoveElements.push(animateResize);

                 sourceNodes[3].setAttributeNS(null, 'id', 'bor.3' + nodeFrom.id);
                 animateResize = createAnimateElement('res.1.' + sourceNodes[3].id, sourceNodes[3].id, 'width', widthFrom, widthTo, '1');
                 animateMoveElements.push(animateResize);

                 sourceNodes[4].setAttributeNS(null, 'id', 'bor.4' + nodeFrom.id);
                 animateResize = createAnimateElement('res.1.' + sourceNodes[4].id, sourceNodes[4].id, 'height', heightFrom, heightTo, '1');
                 animateMoveElements.push(animateResize);*/
            }
            else if (changeBorder) {
                borderSource.setAttributeNS(null, 'id', 'bor.' + nodeFrom.id);
                animateResize = createAnimateElement('res.1.' + borderSource.id, borderSource.id, 'height', heightFrom, heightTo, '1');
                animateMoveElements.push(animateResize);
            }
        }
        if (widthFrom != widthTo) {
            rectSource.setAttributeNS(null, 'id', 'rect.' + nodeFrom.id);
            var animateResize = createAnimateElement('res.2.' + rectSource.id, rectSource.id, 'width', widthFrom, widthTo, '1');
            animateMoveElements.push(animateResize);

            if (changeBorder) {
                borderSource.setAttributeNS(null, 'id', 'bor.' + nodeFrom.id);
                animateResize = createAnimateElement('res.2.' + borderSource.id, borderSource.id, 'width', widthFrom, widthTo, '1');
                animateMoveElements.push(animateResize);
            }
        }

    }
    else {
        /* TODO: PATH */
    }
}

/*----------------------------------------------------------------------------------*/

/*----- creates animation element to move an edge -----*/
function createMoveEdgeAnimation(edgeFrom, edgeTo, ending) {
    var classesFrom = edgeFrom.getAttribute('class').split(' ');
    var classesTo = edgeTo.getAttribute('class').split(' ');

    var edgeFromPath = edgeFrom.getElementsByTagName('path').item(0);
    var edgeToPath = edgeTo.getElementsByTagName('path').item(0);
    var edgeFromArrow = edgeFrom.getElementsByTagName('path').item(1);
    var edgeToArrow = edgeTo.getElementsByTagName('path').item(1);

    edgeFromPath.setAttributeNS(null, 'id', 'e.g' + sourceNo + '.' + ending);
    var pathTo = edgeToPath.getAttributeNS(null, 'd');
    var pathFrom = edgeFromPath.getAttributeNS(null, 'd');

    /*----- true: changes points of path so that the path wouldn't rotate -----*/

    var edgeFromEnding = edgeFrom.id.split('.');
    if (ending != (edgeFromEnding[3] + '.' + edgeFromEnding[4]) && edgeFromArrow == null) {
        var newPath = pathTo.split(' ');
        var lastLetter = newPath[newPath.length - 2].substr(0, 1);
        pathTo = 'M' + newPath[newPath.length - 2].substr(1, newPath[newPath.length - 2].length) + ' ' + newPath[newPath.length - 1];

        for (var k = newPath.length - 3; k > 1; k--) {
            pathTo += ' ' + dAttr[k - 1] + ' ' + dAttr[k] + ' ';
        }

        pathTo += ' ' + lastLetter + newPath[0].substr(1, newPath[0].length) + ' ' + newPath[1];
    }

    /* ensure that both paths have same number of bends */

    var bendsTo = pathTo.split('L');
    var bendsFrom = pathFrom.split('L');

    if (bendsTo.length != bendsFrom.length) {
        var diff = bendsTo.length - bendsFrom.length;

        while (diff > 0) {
            pathFrom += ' L' + bendsFrom[bendsFrom.length - 1];
            diff--;
        }
        edgeFromPath.setAttributeNS(null, 'd', pathFrom);

        while (diff < 0) {
            pathTo += ' L' + bendsTo[bendsTo.length - 1];
            diff++;
        }
        edgeToPath.setAttributeNS(null, 'd', pathTo);
    }

    /*----- Check if edge has to be moved -----*/
    if (pathFrom == pathTo) {
        return;
    }

    var animatePath = createAnimateElement('move.' + edgeFromPath.id, edgeFromPath.id, 'd', pathFrom, '' + pathTo, '1');
    animateMoveElements.push(animatePath);

}

/*----------------------------------------------------------------------------------*/

/*----- creates animation to change color, opacity and stroke -----*/

/* note: yFiles exports at least one element to describe the shape
   (border) if the node is transparent (checked in Visone properties-dialogue),
   else there are two elements (one for the node and one for the border
   whereby the border is the second element) even the node has opacity = 0*/
  
function createColorNodeAnimation(nodeFrom, nodeTo){
       
    var fillOpacityFrom = '1';
    var fillRGBFrom = null;
    var strokeFrom = null;
    var strokeOpacityFrom = '1';
    var strokeWidthFrom = null;
    
    var fillOpacityTo = '1';
    var fillRGBTo = null;
    var strokeTo = null;
    var strokeOpacityTo = '1';
    var strokeWidthTo = '0';
    
    var innerNodeFrom = null;
    var borderFrom = null;
    var innerNodeTo = null;
    var borderTo = null;
    
    var tagName = null;
    var tagNameSource = nodeFrom.getAttribute('class').split(' ')[1];
    var tagNameTarget = nodeTo.getAttribute('class').split(' ')[1];

    if (tagNameSource == 'ellipse') {
        tagName = 'ellipse';
    }
    else if (tagNameSource == 'rectangle' || tagNameSource == '3Drectangle') {
        tagName = 'rect';
    }
    else {
        tagName = 'path';
    }
    
    var elementsFrom = nodeFrom.getElementsByTagName(tagName);

    if (tagNameTarget == 'ellipse') {
        tagName = 'ellipse';
    }
    else if (tagNameTarget == 'rectangle' || tagNameTarget == '3Drectangle') {
        tagName = 'rect';
    }
    else {
        tagName = 'path';
    }
    
    var elementsTo = nodeTo.getElementsByTagName(tagName);
    
    /*checks whether node consists of inner node and border or just border*/
    if(elementsFrom.length == 2){
        /* get attributes of inner node and set id */
        innerNodeFrom = elementsFrom.item(0).parentNode;
        var op = innerNodeFrom.getAttributeNS(null, "fill-opacity");
        if(op != null){
            fillOpacityFrom = op;
        }
        fillRGBFrom = innerNodeFrom.getAttributeNS(null, "fill");
        
        innerNodeFrom.setAttributeNS(null, 'id', 'in.'+nodeFrom.id);
        
        /* get attributes of border and set id */
        borderFrom = elementsFrom.item(1);
        var id = 'bor.'+nodeFrom.id;
        
        if(borderFrom.getAttributeNS(null, 'stroke') == null){
            borderFrom = borderFrom.parentNode;
            strokeOpacityFrom = borderFrom.getAttributeNS(null, 'stroke-opacity');
            id = 'bor2.'+nodeFrom.id;
        }
        
        strokeFrom = borderFrom.getAttributeNS(null, 'stroke');
        strokeWidthFrom = borderFrom.getAttributeNS(null, 'stroke-width');
        
        borderFrom.setAttributeNS(null, 'id', id);
    }
    else {
        /* get attributes of border and set id */
        borderFrom = elementsFrom.item(0);
        var id = 'bor.'+nodeFrom.id;
        
        if(borderFrom.getAttributeNS(null, 'stroke') == null){
            borderFrom = borderFrom.parentNode;
            strokeOpacityFrom = borderFrom.getAttributeNS(null, 'stroke-opacity');
            id = 'bor2.'+nodeFrom.id;
        }
        
        strokeFrom = borderFrom.getAttributeNS(null, 'stroke');
        strokeWidthFrom = borderFrom.getAttributeNS(null, 'stroke-width');
        
        borderFrom.setAttributeNS(null, 'id', id);
    }
    
    /*checks whether node consists of inner node and border or just border*/
    if(elementsTo.length == 2){
        /* get attributes of inner node and set id */
        innerNodeTo = elementsTo.item(0).parentNode;
        var op = innerNodeTo.getAttributeNS(null, "fill-opacity");
        if(op != null){
            fillOpacityTo = op;
        }
        fillRGBTo = innerNodeTo.getAttributeNS(null, "fill");
        
        innerNodeTo.setAttributeNS(null, 'id', 'in.'+nodeTo.id);
        
        /* get attributes of border and set id */
        borderTo = elementsTo.item(1);
        
        if(borderTo.getAttributeNS(null, 'stroke') == null){
            borderTo = borderTo.parentNode;
            strokeOpacityTo = borderTo.getAttributeNS(null, 'stroke-opacity');
        }
        
        strokeTo = borderTo.getAttributeNS(null, 'stroke');
        strokeWidthTo = borderTo.getAttributeNS(null, 'stroke-width');
    }
    else {
        /* get attributes of border and set id */
        borderTo = elementsTo.item(0);
        
        if(borderTo.getAttributeNS(null, 'stroke') == null){
            borderTo = borderTo.parentNode;
            strokeOpacityTo = borderTo.getAttributeNS(null, 'stroke-opacity');
        }
        
        strokeTo = borderTo.getAttributeNS(null, 'stroke');
        strokeWidthTo = borderTo.getAttributeNS(null, 'stroke-width');
    }
    
    if(fillOpacityFrom != fillOpacityTo){
        var animateFillOpacity = createAnimateElement('fillOp.' + innerNodeFrom.id, innerNodeFrom.id, 'fill-opacity', fillOpacityFrom, fillOpacityTo, '1');
        animateMoveElements.push(animateFillOpacity);
    }
    
    if(fillRGBFrom != fillRGBTo && !(fillRGBFrom == null || fillRGBTo == null)){
        var animateFillRGB = createAnimateElement('fillRGB.' + innerNodeFrom.id, innerNodeFrom.id, 'fill', fillRGBFrom, fillRGBTo, '1');
        animateMoveElements.push(animateFillRGB);
    }
    
    if(strokeWidthFrom != strokeWidthTo && !(strokeWidthFrom == null || strokeWidthTo == null)){
        var animateStrokeWidth = createAnimateElement('width.' + borderFrom.id, borderFrom.id, 'stroke-width', strokeWidthFrom, strokeWidthTo, '1');
        animateMoveElements.push(animateStrokeWidth);
    }
    
    if(strokeFrom != strokeTo){
        var animateStroke = createAnimateElement('stroke.' + borderFrom.id, borderFrom.id, 'stroke', strokeFrom, strokeTo, '1');
        animateMoveElements.push(animateStroke);
    }
    
    if(strokeOpacityFrom != strokeOpacityTo && !(strokeOpacityFrom == null || strokeOpacityTo == null)){
        var animateStrokeOpacity = createAnimateElement('strokeOp.' + borderFrom.id, borderFrom.id, 'opacity', strokeOpacityFrom, strokeOpacityTo , '1');
        animateMoveElements.push(animateStrokeOpacity);
    }
    
    
}

/*----------------------------------------------------------------------------------*/

/*----- Moves nodes and edges of the source-graph to the target-graph -----*/
function move() {
    for (var i = 0; i < animateMoveElements.length; i++) {
        animateMoveElements[i].beginElementAt(beginElementDelay);
    }

    setTimeout(fadeIn, 1000 * (duration + beginElementDelay));
}

/*----------------------------------------------------------------------------------*/

/*----- shows all elements of the target graph -----*/
function fadeIn() {
    for (var i = 0; i < nodesTo.length; i++) {
        var nodeTo = nodesTo.item(i);
        var nodeToId = nodeTo.id;

        if (nodeToId == "empty") {
            continue;
        }

        var temp = nodeToId.split('.');
        var nodeId = temp[3];
        
        for(k=4; k<temp.length; k++){
            nodeId += '.'+temp[k];
        }
        
        var str = 'y.node.g' + sourceNo + '.' + nodeId;
        var nodeFrom = document.getElementById(str);

        if (nodeFrom == null || startLooping) {           
            var animateShowNode = createAnimateElement('show.' + nodesTo.item(i).id, nodesTo.item(i).id, 'opacity', '0', '1', '1');
            animateShowElements.push(animateShowNode);
        }
        else {
            nodeFrom.setAttributeNS(null, 'opacity', '0');
            nodesTo.item(i).setAttributeNS(null, 'opacity', '1');
        }
    }

    for (var i = 0; i < edgesTo.length; i++) {
        var edgeTo = edgesTo.item(i);
        var edgeToId = edgeTo.id;

        if (edgeToId == "empty") {
            continue;
        }

        var temp = edgeToId.split('.');
        var edgeId = temp[4];
        
        for(k=5; k<temp.length; k++){
            edgeId += '.'+temp[k];
        }
        
        var str = 'y.edge.g' + sourceNo + '.' + temp[3] + '.' + edgeId;
        var edgeFrom = document.getElementById(str);

        if (edgeFrom == null || startLooping) {
            if(highlight && !startLooping){
                var edgeGroup = edgeTo.childNodes[1];
                edgeColors.push(edgeGroup.getAttributeNS(null, 'stroke'));
                changedEdges.push(edgeGroup);
                edgeGroup.setAttributeNS(null, 'stroke', fadeInColor);
            }
            
            var animateShowEdge = createAnimateElement('show.' + edgesTo.item(i).id, edgesTo.item(i).id, 'opacity', '0', '1', '1');
            animateShowElements.push(animateShowEdge);
        }
        else {
            edgeFrom.setAttributeNS(null, 'opacity', '0');
            edgesTo.item(i).setAttributeNS(null, 'opacity', '1');
        }
    }

    for (var i = 0; i < animateShowElements.length; i++) {
        animateShowElements[i].beginElementAt(beginElementDelay);
    }

    if (animateShowElements.length == 0) {
        setTimeout(switchGraph, 0);
    }
    else {
        setTimeout(switchGraph, 1000 * (duration + beginElementDelay));
    }
};

/*----------------------------------------------------------------------------------*/

function switchGraph() { 
    for (var i = 0; i < nodesTo.length; i++) {
        nodesTo.item(i).setAttributeNS(null, 'opacity', '1');
    }
    for (var i = 0; i < edgesTo.length; i++) {
        edgesTo.item(i).setAttributeNS(null, 'opacity', '1');
    }
    for (var i = 0; i < nodesFrom.length; i++) {
        nodesFrom.item(i).setAttributeNS(null, 'opacity', '0');
    }
    for (var i = 0; i < edgesFrom.length; i++) {
        edgesFrom.item(i).setAttributeNS(null, 'opacity', '0');
    }
    for(var i = 0; i < edgeColors.length; i++) {
        changedEdges[i].setAttributeNS(null, 'stroke', edgeColors[i]);
    }
    
    sourceNo = targetNo;
    targetNo++;
    
    if(controls){
        changeText(targetNo);
        var thumbMove = (targetNo-1) * graphSliderStep + 1;
        graphThumb.setAttributeNS(null, 'transform', 'translate('+thumbMove+',-10)');
    } 
    
    if(playing){
        setTimeout(startHelp, 1000 + 1000 * delay);
        startLooping = false;
    }
    else{
        run = false;
        deleteAnimations();
        
        changeButtonColor(true, true); 
    }
}

/*----------------------------------------------------------------------------------*/

/*----- updates the number of the shown graph -----*/
function changeText(number){
    var textField = document.getElementById('textField');
    textField.textContent = number+'/'+noOfGraphs;
}

/*----------------------------------------------------------------------------------*/

/*----- Creates a button -----*/
function createButton(id, onclick, cx, cy) {
    var button = document.createElementNS('http://www.w3.org/2000/svg', 'g');
    button.setAttributeNS(null, 'id', id);
    button.setAttributeNS(null, 'onclick', onclick);
    
    var circ = document.createElementNS('http://www.w3.org/2000/svg', 'circle');
    circ.setAttributeNS(null, 'cy', cy);
    circ.setAttributeNS(null, 'cx', cx);
    circ.setAttributeNS(null, 'r', buttonRadius);
    circ.setAttributeNS(null, 'fill', buttonColor);
    circ.setAttributeNS(null, 'opacity', '1');
    circ.setAttributeNS(null, 'stroke', 'none');
    
    button.appendChild(circ);
    
    return button;
}

/*----------------------------------------------------------------------------------*/

/*----- Creates a path used as buttonsymbol -----*/
function createButtonPath(startX, startY, d, opacity) {
    var path = document.createElementNS('http://www.w3.org/2000/svg', 'path');
    path.setAttributeNS(null, 'd',d);
    path.setAttributeNS(null, 'transform', 'translate('+startX+','+startY+')');
    path.setAttributeNS(null, 'fill', 'white');
    path.setAttributeNS(null, 'stroke', 'none');
    path.setAttributeNS(null, 'opacity', opacity);
    
    return path;
}

/*----------------------------------------------------------------------------------*/

/*----- Creates an animate element needed for animations -----*/
function createAnimateElement(id, href, attributeName, from, to, alt_duration) {
    var animateElement = document.createElementNS("http://www.w3.org/2000/svg", "animate");

    animateElement.setAttributeNS(null, 'id', '' + id);
    animateElement.setAttributeNS('http://www.w3.org/1999/xlink', 'href', '#' + href);
    animateElement.setAttributeNS(null, 'attributeName', '' + attributeName);
    animateElement.setAttributeNS(null, 'from', '' + from);
    animateElement.setAttributeNS(null, 'to', '' + to);
    animateElement.setAttributeNS(null, 'dur', '' + duration);
    animateElement.setAttributeNS(null, 'fill', 'freeze');
    animateElement.setAttributeNS(null, 'begin', 'undefined');

    document.documentElement.appendChild(animateElement);
    animateElements.push(animateElement);

    return animateElement;
}

/*----- Creates an animateTransform element needed for animations -----*/
function createAnimateTransformElement(id, href, type, attributeName, from, to, additive, alt_duration) {
    var animateTransformElement = document.createElementNS("http://www.w3.org/2000/svg", "animateTransform");

    animateTransformElement.setAttributeNS(null, 'id', '' + id);
    animateTransformElement.setAttributeNS('http://www.w3.org/1999/xlink', 'href', '#' + href);
    animateTransformElement.setAttributeNS(null, 'type', '' + type);
    animateTransformElement.setAttributeNS(null, 'attributeName', '' + attributeName);
    animateTransformElement.setAttributeNS(null, 'from', '' + from);
    animateTransformElement.setAttributeNS(null, 'to', '' + to);
    animateTransformElement.setAttributeNS(null, 'additive', '' + additive);
    animateTransformElement.setAttributeNS(null, 'dur', '' + duration);
    animateTransformElement.setAttributeNS(null, 'fill', 'freeze');
    animateTransformElement.setAttributeNS(null, 'begin', 'undefined');

    document.documentElement.appendChild(animateTransformElement);
    animateElements.push(animateTransformElement);

    return animateTransformElement;
}

/*----------------------------------------------------------------------------------*/
/*---------------------------------Control-Creation---------------------------------*/
/*----------------------------------------------------------------------------------*/

/*----- initialize controls -----*/
function initControls(evt){
    document = evt.target.ownerDocument;
    noOfGraphs = parseFloat(document.documentElement.getAttributeNS(null, 'class'));
    
    document.getElementsByTagName("g").item(0).setAttributeNS(null, 'transform', 'translate(0,100)');
    var docWidth = document.documentElement.getAttributeNS(null, 'width');
    var docHeight = document.documentElement.getAttributeNS(null, 'height');
    
    var docWidth = parseFloat(docWidth);
    
    if(docWidth < 700){
        document.documentElement.setAttributeNS(null, 'width', 700);
        docWidth = 700;
    }
   
    createBar(docHeight, docWidth);
    createButtons(docWidth);
    createTextField(docWidth);
    createSliders(docWidth);
    
    if(noOfGraphs > 1){
        createGraphSlider(docWidth);
    }
    
    createCheckbox(docWidth/2 + xHighlightEdges, yHighlightEdges, 'highlightingBox', 'highlight edges', 'uncheckHighlighting(evt)', 'highlightingCross', 1);
    createCheckbox(docWidth/2 + xLoop, yLoop, 'loopBox', 'loop animation', 'uncheckLoop(evt)', 'loopCross', 0);
    createInputField(docWidth);
    insertVisone();
    
    if(noOfGraphs <= 1){
        changeButtonColor(false, true);
    }
}

/*----------------------------------------------------------------------------------*/

function createBar(docHeight, docWidth){
    var newHeight = parseFloat(docHeight) + 100;
    document.documentElement.setAttributeNS(null, 'height', newHeight);
    
    var bar = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
    bar.setAttributeNS(null, 'height' , '100');
    bar.setAttributeNS(null, 'width', docWidth);
    bar.setAttributeNS(null, 'stroke', 'none');
    bar.setAttributeNS(null, 'fill', barColor);
    bar.setAttributeNS(null, 'y', '0'); 
    document.documentElement.appendChild(bar);
}

/*----------------------------------------------------------------------------------*/

/*----- Creates Buttons -----*/
function createButtons(width){
	var xCoordinate = parseFloat(width)/2 + xPlayPrevNext;
	
	var group = document.createElementNS('http://www.w3.org/2000/svg', 'g');
	group.setAttributeNS(null, 'transform', 'translate('+xCoordinate+','+yPlayPrevNext+')');
    
	var xCircle = 0;
    var playButton = createButton('playButton', 'play(evt)', xCircle, 0);
    
    var startX = xCircle + 9.5;
    var playPath = createButtonPath(startX, 0, playD, '1');
    
    startX = xCircle + 8;
    var startY = 8;
    var stopPath = createButtonPath(startX, startY, stopD, '0');
    playButton.appendChild(playPath);
    playButton.appendChild(stopPath);
    
    xCircle = xCircle + 35;
    var nextButton = createButton('nextButton', 'next(evt)', xCircle, 0);
    
    startX = xCircle + 2;
    var nextPath = createButtonPath(startX, 0, nextD, '1'); 
    nextButton.appendChild(nextPath);
    
    xCircle = xCircle - 70;
    var prevButton = createButton('prevButton', 'previous(evt)', xCircle, 0);
    
    startX = xCircle - 10;
    var prevPath = createButtonPath(startX, 0, prevD, '1');
    prevButton.appendChild(prevPath);
    
    group.appendChild(playButton);
    group.appendChild(nextButton);
    group.appendChild(prevButton);
    document.documentElement.appendChild(group);
}

/*----------------------------------------------------------------------------------*/

/*----- Creates speed- and duration-slider -----*/
function createSliders(width){
    var xCoordinate = parseFloat(width)/2;
    
    var speedSlider = createSlider('speedSlider', xCoordinate+xSpeedSlider, ySpeedSlider, sliderLineD, sliderStrokesD, 81, 0);
    var slow = createTextNode(-13, 35, 'none', font, '14px', 'slow');
    var fast = createTextNode(133, 35, 'none', font, '14px', 'fast');
    var speed = createTextNode(-45, 10, 'none', font, '14px', 'speed');
      
    speedSlider.appendChild(slow);
    speedSlider.appendChild(fast);
    speedSlider.appendChild(speed);
    speedSlider.addEventListener('mousedown', sliderActivate, false);
    speedSlider.addEventListener('mouseup', sliderDeactivate, false);
    
    var delaySlider = createSlider('delaySlider', xCoordinate+xDelaySlider, yDelaySlider, sliderLineD, sliderStrokesD, 0, 0);
    var low = createTextNode(-13, 35, 'none', font, '14px', 'low');
    var high = createTextNode(133, 35, 'none', font, '14px', 'high');
    var delay = createTextNode(-45, 10, 'none', font, '14px', 'delay');
    
    delaySlider.appendChild(low);
    delaySlider.appendChild(high);
    delaySlider.appendChild(delay);
    delaySlider.addEventListener('mousedown', sliderActivate, false);
    delaySlider.addEventListener('mouseup', sliderDeactivate, false);
    
    document.documentElement.appendChild(speedSlider);
    document.documentElement.appendChild(delaySlider);
}

/*----------------------------------------------------------------------------------*/

/*----- creates the graph-slider -----*/
function createGraphSlider(width){
    var x = parseFloat(width)/2 + xGraphSlider;
    
    var strokes = graphSliderStrokesBegin+' ';
    graphSliderStep = graphSliderLength/(noOfGraphs - 1);
    
    for(var i=1; i<noOfGraphs; i++){
        var nextStroke = i*graphSliderStep;
        strokes += 'M'+nextStroke+','+graphStrokeLength+' '+nextStroke+',0 ';
    }
    
    var graphSlider = createSlider('graphSlider', x, yGraphSlider, graphSliderLine, strokes, 0, -10);
    graphThumb = graphSlider.childNodes[3];
    graphSlider.addEventListener('mousedown', sliderActivate, false);
    graphSlider.addEventListener('mouseup', sliderDeactivate, false);
    document.documentElement.appendChild(graphSlider);
}

/*----------------------------------------------------------------------------------*/

/*----- creates a slider -----*/
function createSlider(id, xCoordinate, yCoordinate, line, strokes, defaultThumbPos, yCoordinateThumb){
    var slider = document.createElementNS('http://www.w3.org/2000/svg', 'g');
    slider.setAttributeNS(null, 'id', id);
    slider.setAttributeNS(null, 'transform', 'translate('+xCoordinate+','+yCoordinate+')');
    slider.setAttributeNS(null, 'xoffset', xCoordinate);
    
    var boundingBox = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
    boundingBox.setAttributeNS(null, 'x', -10);
    boundingBox.setAttributeNS(null, 'y', -10);
    boundingBox.setAttributeNS(null, 'height', '40');
    boundingBox.setAttributeNS(null, 'width', '165');
    boundingBox.setAttributeNS(null, 'fill', 'black');
    boundingBox.setAttributeNS(null, 'opacity', '0');
    //boundingBox.setAttributeNS(null, 'onmouseout', 'sliderDeactivate(evt)');
    
    var sliderLine = document.createElementNS('http://www.w3.org/2000/svg', 'path');
    sliderLine.setAttributeNS(null, 'd', line);
    sliderLine.setAttributeNS(null, 'stroke-width', '2');
    
    var sliderStrokes = document.createElementNS('http://www.w3.org/2000/svg', 'path');
    sliderStrokes.setAttributeNS(null, 'd', strokes);
    sliderStrokes.setAttributeNS(null, 'stroke', 'black');
    sliderStrokes.setAttributeNS(null, 'fill', 'none');
    
    var thumb = document.createElementNS('http://www.w3.org/2000/svg', 'g');
    thumb.setAttributeNS(null, 'id', id+'.Thumb');
    thumb.setAttributeNS(null, 'transform', 'translate('+defaultThumbPos+','+yCoordinateThumb+')');
    
    var thumbPath = document.createElementNS('http://www.w3.org/2000/svg', 'path');
    thumbPath.setAttributeNS(null, 'd', thumbD);
    thumbPath.setAttributeNS(null, 'fill', thumbColor);
    thumbPath.setAttributeNS(null, 'stroke', thumbStroke);
    
    thumb.appendChild(thumbPath);
    slider.appendChild(boundingBox);
    slider.appendChild(sliderLine);
    slider.appendChild(sliderStrokes);
    slider.appendChild(thumb);
    
    return slider;
}

/*----------------------------------------------------------------------------------*/

function createCheckbox(x, y, id, text, func, crossId, opacity){
    var checkbox = document.createElementNS('http://www.w3.org/2000/svg', 'g');
    checkbox.setAttributeNS(null, 'id', id);
    checkbox.setAttributeNS(null, 'transform', 'translate('+x+','+y+')');
    
    var text = createTextNode(20, 11, 'none', font, '14px', text);
    
    var box = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
    box.setAttributeNS(null, 'x', 0);
    box.setAttributeNS(null, 'y', 0);
    box.setAttributeNS(null, 'height', '15');
    box.setAttributeNS(null, 'width', '15');
    box.setAttributeNS(null, 'fill', 'white');
    box.setAttributeNS(null, 'opacity', '1');
    box.setAttributeNS(null, 'stroke', 'none');
    box.setAttributeNS(null, 'onclick', func);
    
    var cross = document.createElementNS('http://www.w3.org/2000/svg', 'path');
    cross.setAttributeNS(null, 'd', 'M2,2 L13,13 M2,13 L13,2');
    cross.setAttributeNS(null, 'fill', 'none');
    cross.setAttributeNS(null, 'stroke', 'black');
    cross.setAttributeNS(null, 'opacity', opacity);
    cross.setAttributeNS(null, 'id', 'highlightCross');
    cross.setAttributeNS(null, 'onclick', 'uncheckHighlighting(evt)');
    cross.setAttributeNS(null, 'id', crossId);
    
    
    checkbox.appendChild(box);
    checkbox.appendChild(text);
    checkbox.appendChild(cross);
    
    document.documentElement.appendChild(checkbox);
}

/*----------------------------------------------------------------------------------*/

/*----- creates a text field, that shows the number of the shown graph -----*/
function createTextField(width){
    var y = yNoOfGraphsField;
    var x = parseFloat(width)/2 + xNoOfGraphsField;
    var text = createTextNode(x, y, 'none', font, '16px', '1/'+noOfGraphs);

    text.setAttributeNS(null, 'id', 'textField');
    document.documentElement.appendChild(text);
}

/*----------------------------------------------------------------------------------*/

/*----- creates a TextNode -----*/
function createTextNode(x, y, stroke, font, fontsize, text){
    var textElem = document.createElementNS('http://www.w3.org/2000/svg', 'text');
    textElem.setAttributeNS(null, 'x', x);
    textElem.setAttributeNS(null, 'y', y);
    textElem.setAttributeNS(null, 'stroke', stroke);
    textElem.setAttributeNS(null, 'font-family', font);
    textElem.setAttributeNS(null, 'font-size', fontsize);
    textElem.setAttributeNS(null, 'pointer-events', 'none');
    
    var textNode = document.createTextNode(text);
    textElem.appendChild(textNode);
    
    return textElem;
}

/*----------------------------------------------------------------------------------*/

/*----- creates an input with button -----*/
function createInputField(width){
    var x = parseFloat(width)/2 + xJumpTo;
    var y = yJumpTo;
    var rectWidth = 60;
    var rectHeight = 25;
    var input = createInputNode(x, y);
    x += -60;
    y += 7;
    var button = createJumpButton('jumpButton', 'jumpToEvent(evt)', x, y, rectWidth, rectHeight);
    
    document.documentElement.appendChild(button);
    document.documentElement.appendChild(input);
}

/*----------------------------------------------------------------------------------*/

/*----- creates a input as foreignObject -----*/
function createInputNode(x, y){
    var foreignElem = document.createElementNS('http://www.w3.org/2000/svg', 'foreignObject');
    foreignElem.setAttributeNS(null, 'x', x);
    foreignElem.setAttributeNS(null, 'y', y);
    foreignElem.setAttributeNS(null, 'width', '50');
    foreignElem.setAttributeNS(null, 'height', '50');
    
    var bodyElem = document.createElementNS('http://www.w3.org/1999/xhtml', 'body');
    bodyElem.setAttribute('xmlns', 'http://www.w3.org/1999/xhtml');
    bodyElem.setAttributeNS(null, 'style', 'position:fixed;');
    
    var inputElem = document.createElementNS('http://www.w3.org/1999/xhtml', 'input');
    inputElem.setAttributeNS(null, 'type', 'text');
    inputElem.setAttributeNS(null, 'size', '2');
    inputElem.setAttributeNS(null, 'id', 'inputField');

    bodyElem.appendChild(inputElem);
    foreignElem.appendChild(bodyElem);
    
    return foreignElem;   
}

/*----------------------------------------------------------------------------------*/

/*----- creates jump-button -----*/
function createJumpButton(id, onclick, x, y, width, height) {
    var button = document.createElementNS('http://www.w3.org/2000/svg', 'g');
    button.setAttributeNS(null, 'id', id);
    button.setAttributeNS(null, 'onclick', onclick);
    
    var rect = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
    rect.setAttributeNS(null, 'y', y);
    rect.setAttributeNS(null, 'x', x);
    rect.setAttributeNS(null, 'width', width);
    rect.setAttributeNS(null, 'height', height);
    rect.setAttributeNS(null, 'fill', buttonColor);
    rect.setAttributeNS(null, 'opacity', '1');
    rect.setAttributeNS(null, 'stroke', 'none');
    rect.setAttributeNS(null, 'rx', '5');
    
    x += 4;
    y += 16;
    var text = createTextNode(x, y, 'none', font, '16px', 'jump to');
     
    button.appendChild(rect);
    button.appendChild(text);
    
    return button;
}

/*----------------------------------------------------------------------------------*/

/*- inserts visone link -*/
function insertVisone(){
    /*
    var link = document.createElementNS('http://www.w3.org/2000/svg', 'a');
    link.setAttributeNS('http://www.w3.org/1999/xlink', 'href', visoneLink);
    link.setAttributeNS(null, 'target', '_blank');
    
    var created = createTextNode(5, 15, 'none', font, '16px', visoneLine1);
    created.setAttributeNS(null, 'opacity', '0.5');
    created.setAttributeNS(null, 'pointer-events', 'auto');

    var tspan = document.createElementNS('http://www.w3.org/2000/svg', 'tspan');
    tspan.setAttributeNS(null, 'y', '30');  
    tspan.setAttributeNS(null, 'x', '17');

    var visone = document.createTextNode(visoneLine2);
    
    tspan.appendChild(visone);
    created.appendChild(tspan);
    link.appendChild(created);
    
    document.documentElement.appendChild(link);
    */
    
    visoneCreated = true;
    
    checkForSupport();
}


/*----------------------------------------------------------------------------------*/
/*-------------------------------Control-Actions------------------------------------*/
/*----------------------------------------------------------------------------------*/

/*- activates the slider -*/
function sliderActivate(evt){
    if(sliderActive){
        return;
    }
    
    sliderBoundings = evt.target.parentNode.firstChild;
    thumb = evt.target.parentNode.childNodes[3];
    
    if(evt.target.parentNode.id == 'speedSlider.Thumb' || evt.target.parentNode.id == 'delaySlider.Thumb' || evt.target.parentNode.id == 'graphSlider.Thumb') {
        sliderBoundings = evt.target.parentNode.parentNode.firstChild;
        thumb = evt.target.parentNode.parentNode.childNodes[3];
    }
   
    sliderBoundings.addEventListener('mousemove', sliderMove, null);
    sliderActive = true;
    
    if(sliderBoundings.parentNode.id == 'speedSlider'){
        speedVal = true;
    }
    else if(sliderBoundings.parentNode.id == 'delaySlider'){
        delayVal = true;
    }
    else if(!run){
        graphVal = true;
    }
    else{
        sliderBoundings.removeEventListener('mousemove', sliderMove, null);
        sliderActive = false;
    }
}

/*----------------------------------------------------------------------------------*/

/*- deactivates the slider and changes speed or duration -*/
function sliderDeactivate(evt){
    if(sliderActive){
        sliderBoundings.removeEventListener('mousemove', sliderMove, null);
        sliderActive = false;
        var val;
        
        if(graphVal){
            if(thumbPosition % graphSliderStep < graphSliderStep/2){
                val = thumbPosition - (thumbPosition % graphSliderStep);
                var thumbMove = val+1;
                thumb.setAttributeNS(null, 'transform', 'translate('+thumbMove+',-10)');
            }
            else{
                val = thumbPosition + graphSliderStep - (thumbPosition % graphSliderStep);
                var thumbMove = val+1;
                thumb.setAttributeNS(null, 'transform', 'translate('+thumbMove+',-10)');
            }
            var targetGraph = val/graphSliderStep;
            if(sourceNo != targetGraph){
                jumpToGraph(targetGraph);
            } 
            graphVal = false;
            return;
        }
        
        if(thumbPosition % 10 < 5){
            val = thumbPosition - (thumbPosition % 10);
            var thumbMove = val+1;
            thumb.setAttributeNS(null, 'transform', 'translate('+thumbMove+',0)');
            
        }
        else{
            val = thumbPosition + 10 - (thumbPosition % 10);
            var thumbMove = val+1;
            thumb.setAttributeNS(null, 'transform', 'translate('+thumbMove+',0)');
        }
        
        if(speedVal){
            val = val/10;
            switch(val){
                case 0:
                    slideDuration = 5;
                    break;
                case 1:
                    slideDuration = 4.5;
                    break;
                case 2:
                    slideDuration = 4;
                    break;
                case 3:
                    slideDuration = 3.5;
                    break;
                case 4:
                    slideDuration = 3;
                    break;
                case 5:
                    slideDuration = 2.5;
                    break;
                case 6:
                    slideDuration = 2.25;
                    break;
                case 7:
                    slideDuration = 2;
                    break;
                case 8:
                    slideDuration = 1.75;
                    break;
                case 9:
                    slideDuration = 1.5;
                    break;
                case 10:
                    slideDuration = 1.25;
                    break;
                case 11:
                    slideDuration = 1;
                    break;
                case 12:
                    slideDuration = 0.75;
                    break;
                case 13:
                    slideDuration = 0.5;
                    break;
                case 14:
                    slideDuration = 0;
                    break;
                default:
                    slideDuration = 2;
            }
            
            speedVal = false;
        }
        else{
            val = val/10;
            switch(val){
                case 0:
                    slideDelay = 0;
                    break;
                case 1:
                    slideDelay = 0.5;
                    break;
                case 2:
                    slideDelay = 1;
                    break;
                case 3:
                    slideDelay = 1.5;
                    break;
                case 4:
                    slideDelay = 2;
                    break;
                case 5:
                    slideDelay = 2.5;
                    break;
                case 6:
                    slideDelay = 3;
                    break;
                case 7:
                    slideDelay = 3.5;
                    break;
                case 8:
                    slideDelay = 4;
                    break;
                case 9:
                    slideDelay = 4.5;
                    break;
                case 10:
                    slideDelay = 5;
                    break;
                case 11:
                    slideDelay = 5.5;
                    break;
                case 12:
                    slideDelay = 6;
                    break;
                case 13:
                    slideDelay = 6.5;
                    break;
                case 14:
                    slideDelay = 7;
                    break;
                default:
                    slideDelay = 0;
            }
            slideBeginElementDelay = slideDelay;
            delayVal = false;
        }
        
    } 
}

/*----------------------------------------------------------------------------------*/

/*- moves slider -*/
function sliderMove(evt){
    var slider = evt.target.parentNode;
    var val = evt.clientX - parseFloat(slider.getAttributeNS(null, 'xoffset'));
    if(val >= 0 && val <= 140){
        if(graphVal){
            thumb.setAttributeNS(null, 'transform', 'translate('+val+',-10)');
        }
        else{
            thumb.setAttributeNS(null, 'transform', 'translate('+val+',0)');
        }
        thumbPosition = val;
    }
}

/*----------------------------------------------------------------------------------*/

function uncheckHighlighting(evt){
    var cross = document.getElementById('highlightingCross');
    
    if(highlight){
        cross.setAttributeNS(null, 'opacity', '0');
        highlight = false;
    }
    else{
        cross.setAttributeNS(null, 'opacity', '1');
        highlight = true;
    }
}

/*----------------------------------------------------------------------------------*/

function uncheckLoop(evt){
	var loopCross = document.getElementById('loopCross');
	
	if(loop){
		loopCross.setAttributeNS(null, 'opacity', '0');
		loop = false;
	}
	else{
		loopCross.setAttributeNS(null, 'opacity', '1');
		loop = true;
	}
}

/*----------------------------------------------------------------------------------*/

function checkForSupport(){
    if(supportChecked){
        return;
    }
    else{
        if(!document.implementation.hasFeature("TimeControl", "1.1")){
            alert('It seems that your browser doesn\'t support SVG-Animations');
        }
        supportChecked = true;
    }
}
